@@ -67,7 +67,7 @@ static ngx_int_t
 ngx_http_vhost_traffic_status_display_handler_control(ngx_http_request_t *r)
 {
     ngx_int_t                                  size, rc;
-    ngx_str_t                                  type, alpha, arg_cmd, arg_group, arg_zone;
+    ngx_str_t                                  type, alpha, encoded_ch, arg_cmd, arg_group, arg_zone;
     ngx_buf_t                                 *b;
     ngx_chain_t                                out;
     ngx_slab_pool_t                           *shpool;
@@ -125,7 +125,8 @@ ngx_http_vhost_traffic_status_display_handler_control(ngx_http_request_t *r)
 
         if (ngx_http_arg(r, (u_char *) "group", 5, &arg_group) == NGX_OK) {
 
-            if (arg_group.len == 1 && ngx_strncmp(arg_group.data, "*", 1) == 0)
+            if ((arg_group.len == 1 && ngx_strncmp(arg_group.data, "*", 1) == 0)
+                     || (arg_group.len == 3 && ngx_strncasecmp(arg_group.data, (u_char *) "%2A", 3) == 0))
             {
                 control->group = -1;
             }
@@ -134,13 +135,17 @@ ngx_http_vhost_traffic_status_display_handler_control(ngx_http_request_t *r)
             {
                 control->group = NGX_HTTP_VHOST_TRAFFIC_STATUS_UPSTREAM_NO;
             }
-            else if (arg_group.len == 14
+            else if ((arg_group.len == 14
                      && ngx_strncasecmp(arg_group.data, (u_char *) "upstream@alone", 14) == 0)
+                     || (arg_group.len == 16
+                     && ngx_strncasecmp(arg_group.data, (u_char *) "upstream%40alone", 16) == 0))
             {
                 control->group = NGX_HTTP_VHOST_TRAFFIC_STATUS_UPSTREAM_UA;
             }
-            else if (arg_group.len == 14
+            else if ((arg_group.len == 14
                      && ngx_strncasecmp(arg_group.data, (u_char *) "upstream@group", 14) == 0)
+                     || (arg_group.len == 16
+                     && ngx_strncasecmp(arg_group.data, (u_char *) "upstream%40group", 16) == 0))
             {
                 control->group = NGX_HTTP_VHOST_TRAFFIC_STATUS_UPSTREAM_UG;
             }
@@ -171,6 +176,46 @@ ngx_http_vhost_traffic_status_display_handler_control(ngx_http_request_t *r)
                               "display_handler_control::copy_str() failed");
             }
 
+            ngx_str_set(&encoded_ch, "%2A");
+
+            rc = ngx_http_vhost_traffic_status_replace_strc(control->zone, &encoded_ch, '*');
+            if (rc != NGX_OK) {
+                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                              "display_handler_control::replace_strc() failed");
+            }
+
+            ngx_str_set(&encoded_ch, "%2a");
+
+            rc = ngx_http_vhost_traffic_status_replace_strc(control->zone, &encoded_ch, '*');
+            if (rc != NGX_OK) {
+                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                              "display_handler_control::replace_strc() failed");
+            }
+
+            ngx_str_set(&encoded_ch, "%3A");
+
+            rc = ngx_http_vhost_traffic_status_replace_strc(control->zone, &encoded_ch, ':');
+            if (rc != NGX_OK) {
+                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                              "display_handler_control::replace_strc() failed");
+            }
+
+            ngx_str_set(&encoded_ch, "%3a");
+
+            rc = ngx_http_vhost_traffic_status_replace_strc(control->zone, &encoded_ch, ':');
+            if (rc != NGX_OK) {
+                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                              "display_handler_control::replace_strc() failed");
+            }
+
+            ngx_str_set(&encoded_ch, "%40");
+
+            rc = ngx_http_vhost_traffic_status_replace_strc(control->zone, &encoded_ch, '@');
+            if (rc != NGX_OK) {
+                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                              "display_handler_control::replace_strc() failed");
+            }
+
             (void) ngx_http_vhost_traffic_status_replace_chrc(control->zone, '@',
                        NGX_HTTP_VHOST_TRAFFIC_STATUS_KEY_SEPARATOR);
 
@@ -277,7 +322,7 @@ static ngx_int_t
 ngx_http_vhost_traffic_status_display_handler_default(ngx_http_request_t *r)
 {
     size_t                                     len;
-    u_char                                    *o, *s, *p;
+    u_char                                    *o, *s, *p, *last_symbol;
     ngx_str_t                                  uri, euri, type;
     ngx_int_t                                  size, format, rc;
     ngx_buf_t                                 *b;
@@ -341,7 +386,9 @@ ngx_http_vhost_traffic_status_display_handler_default(ngx_http_request_t *r)
             }
         }
 
-        if ((s = (u_char *) ngx_strchr(++s, '/')) == NULL) {
+        last_symbol = r->uri.data + r->uri.len;
+
+        if ((s = (u_char *) ngx_strlchr(++s, last_symbol, '/')) == NULL) {
             break;
         }
 
